//////////////////////////////////////////////////////////////////////
// Trigger.cpp
// -------------------------------------------------------------------
// This module will automatically execute commands when a specific packet is
// received or sent.  Information in the packets can be exchanged via the XX
// specification.
//
// e.g. 
//		.trigger set r 9dXXXXXX******************** send 17XXXXXX
//
// This will automatically send packet 17 with the values from packet 
// packet 9d in bytes 2-4.
//
//	0.02 bootyjuice	Original version
//	0.03 bootyjuice	Fixed xx, yy, and zz buffer issue.
//	0.04 bootyjuice	Added multiple command support.
//  0.05 bootyjuice Added block command.
//  0.06 bootyjuice Added sub command and added uu,vv,ww buffers.
//
// bootyjuice
//////////////////////////////////////////////////////////////////////
#include "..\ClientCore.cpp"	  // Include the backend of the module

// Functions as they appear in the source...
BOOL PRIVATE OnGameCommandLoad(char** argv, int argc);
BOOL PRIVATE OnGameCommandSave(char** argv, int argc);
BOOL PRIVATE OnGameCommandSet(char** argv, int argc);
BOOL PRIVATE OnGameCommandList(char** argv, int argc);
BOOL PRIVATE OnGameCommandClear(char** argv, int argc);

// Some global stuff
FINGERPRINTSTRUCT fps;

char	CurrentSection[64]={"default"};

char uu[256] = {0};
char vv[256] = {0};
char ww[256] = {0};
char xx[256] = {0};
char yy[256] = {0};
char zz[256] = {0};

char cCommandSeparator = ';';

//////////////////////////////////////////////////////////////////////
// CLIENTINFO
// -------------------------------------------------------------------
// You *need* this, don't ask why :)
//
// 0.01 2001-10-25 First version
// 0.02 2001-12-28 Generalized packet parameters.
//////////////////////////////////////////////////////////////////////
DWORD	ModuleVersion=MAKELONG(0,6);
char	ModuleAuthor[]="bootyjuice";
char	ModuleWebsite[]="home.attbi.com/~bootyjuice";
char	ModuleDescription[]="Trigger module - executes commands when a packet is sent or received.";
char	ModuleEmail[]="bootyjuice";

typedef struct trigger
{
	char* RS;
	char* triggerPacket;
	char* Command;
} TRIGGER;

TRIGGER triggers[256];
int triggerCount=0;

//////////////////////////////////////////////////////////////////////
// MODULECOMMANDSTRUCT ModuleCommands[]
// -------------------------------------------------------------------
// To add your own commands, just add them to this list like:
//
//	{
//		"name"			// Name of the command
//		pFunction		// The function that handles command
//		"usage"			// Help text for the command
//	}
// 
// Functions should use following convention...
// BOOL PRIVATE OnGameCommandName(char** argv, int argc)
// ...and return TRUE on success or FALSE to show usage of the command.
//
// To use your commands in the game, just type 
// .modulename <command> [arguments]
//////////////////////////////////////////////////////////////////////

MODULECOMMANDSTRUCT ModuleCommands[]=
{
	{	"load",
		OnGameCommandLoad,
		"load [section]c0 Loads triggers in [section] from trigger.ini."
	},
	{	"save",
		OnGameCommandSave,
		"save [section]c0 Saves current triggers to <section> in trigger.ini."
	},
	{
		"set",
		OnGameCommandSet,
		"set <r|s> <id> [command1 [;command2] [;command3] ... ]>\n"
		"Sets a trigger on packet <id> that will run <command> when processed.\n"
		"Use buffers (UU,VV,WW,XX,YY,ZZ) to store values e.g. trigger set s 19XXXXXXXX say Item ID is XXXXXXXX\n"
		"Use 'block' to prevent packet from being sent/received e.g. trigger set s 19 block\n"
		"Use 'sub' to change one packet into another e.g. trigger set s 19 sub 1901000000\n"
	},
	{
		"list",
		OnGameCommandList,
		"listc0 Lists all triggers currently active."
	},
	{	"clear",
		OnGameCommandClear,
		"clearc0 Removes all triggers."
	},
	{	// The 'help' command is implemented in ClientCore.cpp and 
		// should not need to be changed.
		"help",
		OnGameCommandHelp,
		"helpc0 List commands available in this module.\n"
		"<command> helpc0 Shows detailed help for <command> in this module."
	},
	{NULL}	// No more commands
};

//////////////////////////////////////////////////////////////////////
// substituteValues
// -------------------------------------------------------------------
// Exchanges XX, YY, and ZZ values in a string.
//////////////////////////////////////////////////////////////////////
BOOL substituteValues(char* sTemp)
{
	// Plug in the values for XX into the string.
	while(strchr(sTemp,'U') != NULL)
	{
		strncpy(strchr(sTemp,'U'), uu, strlen(uu));
	}

	// Plug in the values for XX into the string.
	while(strchr(sTemp,'V') != NULL)
	{
		strncpy(strchr(sTemp,'V'), vv, strlen(vv));
	}

		// Plug in the values for XX into the string.
	while(strchr(sTemp,'W') != NULL)
	{
		strncpy(strchr(sTemp,'W'), ww, strlen(ww));
	}

	// Plug in the values for XX into the string.
	while(strchr(sTemp,'X') != NULL)
	{
		strncpy(strchr(sTemp,'X'), xx, strlen(xx));
	}

	// Plug in the values for YY into the string.
	while(strchr(sTemp,'Y') != NULL)
	{
		strncpy(strchr(sTemp,'Y'), yy, strlen(yy));
	}

	// Plug in the values for ZZ into the string.
	while(strchr(sTemp,'Z') != NULL)
	{
		strncpy(strchr(sTemp,'Z'), zz, strlen(zz));
	}

	return true;
}

//////////////////////////////////////////////////////////////////////
// OnClientStart
// -------------------------------------------------------------------
// Runs *once* before client is loaded. Return FALSE to prevent from
// loading.
//////////////////////////////////////////////////////////////////////
BOOL EXPORT OnClientStart()
{
	// Initiate list
	char *argv[3];
	char argc=3;
	argv[0]="";
	argv[1]="load";
	argv[2]="default";
	OnGameCommandLoad(argv, argc);
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnClientCommandLoad
// -------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandLoad(char** argv, int argc)
{
	char* triggerTemp = 0;
	char t[1024];
	
	// If command line is not 2 or 3 tokens, something is wrong
	if ((argc < 2) && (argc > 3))
	{
		return FALSE;
	}

	if (argc == 3)
	{
		strcpy(CurrentSection, argv[2]);
	}

	// Notify the user.
	sprintf(t, "Loading triggers from section: c4[%s]",CurrentSection);
	server->GamePrintInfo(t);

	// Clear all the current triggers.
	for(int i = 0; i < triggerCount; i++)
	{
		delete triggers[i].Command;
		triggers[i].Command = NULL;
		delete triggers[i].RS;
		triggers[i].RS = NULL;
		delete triggers[i].triggerPacket;
		triggers[i].triggerPacket = NULL;
	}
	triggerCount = 0;

	//char retval[1024] = {0};

	// This module only supports up to 256 triggers.
	for (i=0; i!=256; i++)
	{
		sprintf(t, "%i", i);

		// sprintf(retval, "%s", server->GetHackProfileString("trigger", CurrentSection, t));

		char* retval = server->GetHackProfileString("trigger", CurrentSection, t);

		// If we found anything then process it.
		if(retval[0] != 0)
		{

			// Parse out the parameters.
			triggers[triggerCount].RS = new char[2];
			triggers[triggerCount].RS[0] = retval[0];
			triggers[triggerCount].RS[1] = 0;

			// Determine the location of the next space character
			size_t triggerEnd;
			triggerEnd = strcspn(&retval[2], " ") + 2;

			// Allocate space for the new trigger string.
			// Subtract one according to this formula: +1 for null character - 2 for
			// 2 leading characters in retval before trigger begins = -1.
			triggers[triggerCount].triggerPacket = new char[triggerEnd - 1];

			// Copy the trigger string value in.
			strncpy(triggers[triggerCount].triggerPacket, &retval[2], triggerEnd - 2);

			// Add a null termination character since strncpy doesn't always do it.
			triggers[triggerCount].triggerPacket[triggerEnd - 2] = 0;

			// Copy the command.
			triggers[triggerCount].Command = new char[strlen(retval) - triggerEnd];
			strcpy(triggers[triggerCount].Command,&retval[triggerEnd + 1]);

			// Increment the trigger count.
			triggerCount++;

			/*
			delete retval;  // This crashes the game.  I don't know why.
			*/
		}
	}

	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnClientCommandSave
// -------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandSave(char** argv, int argc)
{
	// If command line is not 2 or 3 tokens, something is wrong
	if ((argc < 2) && (argc > 3))return FALSE;

	char t[1024];

	if (argc == 3)
	{
		strcpy(CurrentSection, argv[2]);
	}

	sprintf(t, "Saving triggers to section: c4[%s]",CurrentSection);
	server->GamePrintInfo(t);

	char triggerTemp[1024] = {0};
	char triggerNumber[256] = {0};

	for (int i = 0; i < triggerCount; i++)
	{
		// Format the strings to write out.
		sprintf(triggerTemp, "%s %s %s", triggers[i].RS, triggers[i].triggerPacket, triggers[i].Command);
		sprintf(triggerNumber, "%i", i);

		// Write out the trigger to the ini file.
		server->SetHackProfileString("trigger", CurrentSection, triggerNumber, triggerTemp);
	}

	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnClientCommandSet
// -------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandSet(char** argv, int argc)
{
	// If command line is shorter than 4 tokens, something is wrong
	if (argc<4) return FALSE;

	// Check to make sure 3rd parameter is <r|s>
	if ( stricmp(argv[2], "r") && stricmp(argv[2], "s"))
	{
		return FALSE;
	}

	// Create space for working with strings.
	char t[1024];
	
	triggers[triggerCount].RS = new char[strlen(argv[2]) + 1];
	strcpy(triggers[triggerCount].RS, argv[2]);

	triggers[triggerCount].triggerPacket = new char[strlen(argv[3]) + 1];
	strcpy(triggers[triggerCount].triggerPacket, argv[3]);

	// Copy the command into the memory structure.
	t[0]=0;

	// Is there a command to copy?
	if (argc >= 5)
	{
		strcat(t, argv[4]);

		for (int i=5; i<argc; i++)
		{
			strcat(t, " ");
			strcat(t, argv[i]);
		}
	}

	triggers[triggerCount].Command=new char[strlen(t)+1];
	strcpy(triggers[triggerCount].Command,t);

	triggerCount++;

	server->GamePrintInfo("Ok");
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnClientCommandList
// -------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandList(char** argv, int argc)
{
	char triggerInfo[1024];

	sprintf(triggerInfo,"");
	
	// If command line is not 2 tokens, something is wrong
	if (argc != 2) return FALSE;

	// Check for a blank list.
	if ( triggerCount == 0 ) 
	{
		server->GamePrintInfo("No triggers currently active.");
	}

	for(int i = 0; i < triggerCount; i++)
	{

		// Format the information.
		sprintf(triggerInfo, "Trigger: %s %s Command: %s", triggers[i].RS, triggers[i].triggerPacket, triggers[i].Command);
		
		// Print the information.
		server->GamePrintInfo(triggerInfo);
	}	
	
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnClientCommandClear
// -------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandClear(char** argv, int argc)
{
	// If command line other than 2 tokens, something is wrong
	if (argc<2 || argc>2)
	{
		return FALSE;
	}

	for(int i = 0; i < triggerCount; i++)
	{
		delete triggers[i].Command;
		triggers[i].Command = NULL;
		delete triggers[i].RS;
		triggers[i].RS = NULL;
		delete triggers[i].triggerPacket;
		triggers[i].triggerPacket = NULL;
	}

	triggerCount = 0;

	uu[0] = vv[0] = ww[0] = xx[0] = yy[0] = zz[0] = 0;

	server->GamePrintInfo("All triggers has been cleared. No triggers are active.");
	
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnGamePacketBeforeReceived
// -------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
DWORD EXPORT OnGamePacketBeforeReceived(BYTE* aPacket, DWORD aLen)
{
	char aPacketString[1024]={0};
	char commandTemp[1024]={0};
	DWORD returnLength;
	char t[1024];
	int i;

	bool packetMatches = true;

	char uuTemp[256] = {0};
	char vvTemp[256] = {0};
	char wwTemp[256] = {0};
	char xxTemp[256] = {0};
	char yyTemp[256] = {0};
	char zzTemp[256] = {0};

	// Initialize returnLength to the length of the packet.
	// If we block the packet send then this will be set to 0.
	returnLength = aLen;

	// Convert packet parameter to a string.
	for(i = 0; i < aLen; i++)
	{
		sprintf(aPacketString, "%s%.2x", aPacketString, aPacket[i]);
	}

	// For each active trigger ...
	for(i = 0; i < triggerCount; i++)
	{
		
		// If this trigger isn't for a received packet then move on.
		if(stricmp(triggers[i].RS,"s") == 0)
		{
			continue;
		}

		packetMatches = true;

		uuTemp[0] = vvTemp[0] = wwTemp[0] = xxTemp[0] = yyTemp[0] = zzTemp[0] = 0;

		// Search through the trigger packet for matching values and info to save.
		for(int j = 0; j < strlen(triggers[i].triggerPacket); j += 2)
		{
			//Check for wildcard characters.
			if(strncmp(&triggers[i].triggerPacket[j],"**",2) == 0)
			{
				continue;
			}
			else if(strncmp(&triggers[i].triggerPacket[j],"UU",2) == 0)
			{
				// Keep this value around.
				sprintf(uuTemp, "%s%.2s", uuTemp, &aPacketString[j]);
			}
			else if(strncmp(&triggers[i].triggerPacket[j],"VV",2) == 0)
			{
				// Keep this value around.
				sprintf(vvTemp, "%s%.2s", vvTemp, &aPacketString[j]);
			}
			else if(strncmp(&triggers[i].triggerPacket[j],"WW",2) == 0)
			{
				// Keep this value around.
				sprintf(wwTemp, "%s%.2s", wwTemp, &aPacketString[j]);
			}
			else if(strncmp(&triggers[i].triggerPacket[j],"XX",2) == 0)
			{
				// Keep this value around.
				sprintf(xxTemp, "%s%.2s", xxTemp, &aPacketString[j]);
			}
			else if(strncmp(&triggers[i].triggerPacket[j],"YY",2) == 0)
			{
				// Keep this value around.
				sprintf(yyTemp, "%s%.2s", yyTemp, &aPacketString[j]);
			}
			else if(strncmp(&triggers[i].triggerPacket[j],"ZZ",2) == 0)
			{
				// Keep this value around.
				sprintf(zzTemp, "%s%.2s", zzTemp, &aPacketString[j]);
			}
			else
			{
				// Check to make sure that this portion of the packet matches.
				if(strncmp(&triggers[i].triggerPacket[j], &aPacketString[j], 2))
				{
					// The packet didn't match so take no action.
					packetMatches = false;
					break;
				}
			}		
		}

		// If the packet matched then save the information and 
		// parse and execute the command.
		if( packetMatches)
		{
			// Store the variable values for later.
			if( strlen(uuTemp) )
			{
				strcpy(uu, uuTemp);
			}

			if( strlen(vvTemp) )
			{
				strcpy(vv, vvTemp);
			}

			if( strlen(wwTemp) )
			{
				strcpy(ww, wwTemp);
			}

			if( strlen(xxTemp) )
			{
				strcpy(xx, xxTemp);
			}
			if( strlen(yyTemp) )
			{
				strcpy(yy, yyTemp);
			}

			if( strlen(zzTemp) )
			{
				strcpy(zz, zzTemp);
			}

			// Make sure there is a command to execute.
			if ( strlen(triggers[i].Command) > 0 )
			{
			
				// Copy the command so we can manipulate it.
				strcpy(commandTemp, triggers[i].Command);

				// Plug in all the values for XX, YY, and ZZ.
				substituteValues(commandTemp);

				// If there are multiple commands to be executed then parse through 
				// and execute all of them.
				char* ptrSeparator = strchr(commandTemp, cCommandSeparator);
				while ( ptrSeparator )
				{
					// Change the command separator to a null character and
					// advance the ptrSeparator one character.
					memset(ptrSeparator++,0,1);

					// Is this the block command?
					if( stricmp(commandTemp, "block") == 0 )
					{

						// This is the block command so do not send/receive the packet.
						sprintf(t, "c1BLOCKED c8RECVc0:");
						for(i = 0; i < aLen; i++)
						{
							sprintf(t, "%s %.2x", t, aPacket[i]);
						}
						server->GamePrintString(t);

						returnLength = 0;
					}
					// Is this the substitute command?
					else if( strnicmp(commandTemp, "sub", 3) == 0 )
					{

						// Sanity check.
						if( strlen(commandTemp) < 5)
						{
							sprintf(t,"c1Errorc0 You must specify a packet to substitute for the original.");
							server->GamePrintString(t);
							return aLen;
						}

						if( strlen(&commandTemp[4])/2 != aLen)
						{
							sprintf(t,"c1Errorc0 You must specify a packet of equal length to substitute for the original.");
							server->GamePrintString(t);
							return aLen;
						}

						// This is the substitute command so 
						// use a different packet in place of the 
						// original one.

						// First, put any XX, YY, or ZZ values in there.
						substituteValues(&commandTemp[4]);
	
						// Now put the alternate packet in place of the real one.
						for( i=aLen-1 ; i >= 0 ; i-- )
						{
							//Convert the packet into binary format from back to front
							// and overwrite the original one.
							char *x="";
							aPacket[i] = (BYTE)strtoul(&commandTemp[4 + i*2], &x, 0x10);

							// Trim the packet from the end backwards so we're
							// only converting one byte at a time.
							commandTemp[4 + i*2] = 0;
						}

						// Inform the user what we've done.
						sprintf(t, "c1SUBSTITUTED c8RECVc0:");
						for(i = 0; i < aLen; i++)
						{
							sprintf(t, "%s %.2x", t, aPacket[i]);
						}
						server->GamePrintInfo(t);

						returnLength = aLen;
					}
					else
					{
						// Execute the game command.
						server->GameCommandLine(commandTemp);
					}

					// Move on to the next command.
					strcpy(commandTemp, ptrSeparator);

					// Search for the next command separator.
					ptrSeparator = strchr(commandTemp, cCommandSeparator);
				}

				// Executed all of the commands except the last one ... so do that
				// one now.
				if( stricmp(commandTemp, "block") == 0 )
				{
					// This is the block command so do not send/receive the packet.
					sprintf(t, "c1BLOCKED c8RECVc0:");
					for(i = 0; i < aLen; i++)
					{
						sprintf(t, "%s %.2x", t, aPacket[i]);
					}
					server->GamePrintString(t);

					returnLength = 0;
				}
				// Is this the substitute command?
				else if( strnicmp(commandTemp, "sub", 3) == 0 )
				{

					// Sanity check.
					if( strlen(commandTemp) < 5)
					{
						sprintf(t,"c1Errorc0 You must specify a packet to substitute for the original.");
						server->GamePrintString(t);
						return aLen;
					}

					if( strlen(&commandTemp[4])/2 != aLen)
					{
						sprintf(t,"c1Errorc0 You must specify a packet of equal length to be substituted for the original.");
						server->GamePrintString(t);
						return aLen;
					}

					// This is the substitute command so 
					// use a different packet in place of the 
					// original one.

					// First, put any XX, YY, or ZZ values in there.
					substituteValues(&commandTemp[4]);

					// Now put the alternate packet in place of the real one.
					for( i=aLen-1 ; i >= 0 ; i-- )
					{
						//Convert the packet into binary format from back to front
						// and overwrite the original one.
						char *x="";
						aPacket[i] = (BYTE)strtoul(&commandTemp[4 + i*2], &x, 0x10);

						// Trim the packet from the end backwards so we're
						// only converting one byte at a time.
						commandTemp[4 + i*2] = 0;
					}

					// Inform the user what we've done.
					sprintf(t, "c1SUBSTITUTED c8RECVc0:");
					for(i = 0; i < aLen; i++)
					{
						sprintf(t, "%s %.2x", t, aPacket[i]);
					}
					server->GamePrintInfo(t);

					returnLength = aLen;
				}
				else
				{
					// Execute the game command.
					server->GameCommandLine(commandTemp);
				}

			}
		}
	}

	return returnLength;
}

//////////////////////////////////////////////////////////////////////
// OnGamePacketBeforeSent
// -------------------------------------------------------------------
//////////////////////////////////////////////////////////////////////
DWORD EXPORT OnGamePacketBeforeSent(BYTE* aPacket, DWORD aLen)
{
	char aPacketString[1024] = {0};
	char commandTemp[1024] = {0};
	DWORD returnLength;
	char t[1024];
	int i;

	bool packetMatches = true;

	char uuTemp[256] = {0};
	char vvTemp[256] = {0};
	char wwTemp[256] = {0};
	char xxTemp[256] = {0};
	char yyTemp[256] = {0};
	char zzTemp[256] = {0};

	// Initialize returnLength to the length of the packet.
	// If we block the packet send then this will be set to 0.
	returnLength = aLen;

	// Convert packet parameter to a string.
	for(i = 0; i < aLen; i++)
	{
		sprintf(aPacketString, "%s%.2x", aPacketString, aPacket[i]);
	}

	// For each active trigger ...
	for(i = 0; i < triggerCount; i++)
	{
		
		// If this trigger isn't for a sent packet then move on.
		if(stricmp(triggers[i].RS,"r") == 0)
		{
			continue;
		}

		packetMatches = true;

		uuTemp[0] = vvTemp[0] = wwTemp[0] = xxTemp[0] = yyTemp[0] = zzTemp[0] = 0;

		// Search through the trigger packet for matching values and info to save.
		for(int j = 0; j < strlen(triggers[i].triggerPacket); j += 2)
		{
			//Check for wildcard characters.
			if(strncmp(&triggers[i].triggerPacket[j],"**",2) == 0)
			{
				continue;
			}
			else if(strncmp(&triggers[i].triggerPacket[j],"UU",2) == 0)
			{
				// Keep this value around.
				sprintf(uuTemp, "%s%.2s", uuTemp, &aPacketString[j]);
			}
			else if(strncmp(&triggers[i].triggerPacket[j],"VV",2) == 0)
			{
				// Keep this value around.
				sprintf(vvTemp, "%s%.2s", vvTemp, &aPacketString[j]);
			}
			else if(strncmp(&triggers[i].triggerPacket[j],"WW",2) == 0)
			{
				// Keep this value around.
				sprintf(wwTemp, "%s%.2s", wwTemp, &aPacketString[j]);
			}
			else if(strncmp(&triggers[i].triggerPacket[j],"XX",2) == 0)
			{
				// Keep this value around.
				sprintf(xxTemp, "%s%.2s", xxTemp, &aPacketString[j]);
			}
			else if(strncmp(&triggers[i].triggerPacket[j],"YY",2) == 0)
			{
				// Keep this value around.
				sprintf(yyTemp, "%s%.2s", yyTemp, &aPacketString[j]);
			}
			else if(strncmp(&triggers[i].triggerPacket[j],"ZZ",2) == 0)
			{
				// Keep this value around.
				sprintf(zzTemp, "%s%.2s", zzTemp, &aPacketString[j]);
			}
			else
			{
				// Check to make sure that this portion of the packet matches.
				if(strncmp(&triggers[i].triggerPacket[j], &aPacketString[j], 2))
				{
					// The packet didn't match so take no action.
					packetMatches = false;
					break;
				}
			}		
		}

		// If the packet matched then save the information and 
		// parse and execute the command.
		if( packetMatches)
		{
			// Store the variable values for later.
			if( strlen(uuTemp) )
			{
				strcpy(uu, uuTemp);
			}

			if( strlen(vvTemp) )
			{
				strcpy(vv, vvTemp);
			}

			if( strlen(wwTemp) )
			{
				strcpy(ww, wwTemp);
			}

			if( strlen(xxTemp) )
			{
				strcpy(xx, xxTemp);
			}

			if( strlen(yyTemp) )
			{
				strcpy(yy, yyTemp);
			}

			if( strlen(zzTemp) )
			{
				strcpy(zz, zzTemp);
			}

			// Make sure there is a command to execute.
			if ( strlen(triggers[i].Command) > 0 )
			{
				// Copy the command so we can manipulate it.
				strcpy(commandTemp, triggers[i].Command);

				// Plug in the values for XX, YY, and ZZ into the command.
				substituteValues(commandTemp);

				// If there are multiple commands to be executed then parse through
				// and execute all of them.
				char* ptrSeparator = strchr(commandTemp, cCommandSeparator);
				while ( ptrSeparator )
				{
					// Change the command separator to a null character.
					memset(ptrSeparator++,0,1);

					// Is this the block command?
					if( stricmp(commandTemp, "block") == 0 )
					{
						// This is the block command so do not send/receive the packet.
						sprintf(t, "c1BLOCKED c9SENTc0:");
						for(i = 0; i < aLen; i++)
						{
							sprintf(t, "%s %.2x", t, aPacket[i]);
						}
						server->GamePrintString(t);

						returnLength = 0;
					}
					// Is this the substitute command?
					else if( strnicmp(commandTemp, "sub", 3) == 0 )
					{

						// Sanity check.
						if( strlen(commandTemp) < 5)
						{
							sprintf(t,"c1Errorc0 You must specify a packet to substitute for the original.");
							server->GamePrintString(t);
							return aLen;
						}

						if( strlen(&commandTemp[4])/2 != aLen)
						{
							sprintf(t,"c1Errorc0 You must specify a packet of equal length to be substituted for the original.");
							server->GamePrintString(t);
							return aLen;
						}

						// This is the substitute command so 
						// use a different packet in place of the 
						// original one.

						// First, put any XX, YY, or ZZ values in there.
						substituteValues(&commandTemp[4]);
	
						// Now put the alternate packet in place of the real one.
						for( i=aLen-1 ; i >= 0 ; i-- )
						{
							//Convert the packet into binary format from back to front
							// and overwrite the original one.
							char *x="";
							aPacket[i] = (BYTE)strtoul(&commandTemp[4 + i*2], &x, 0x10);

							// Trim the packet from the end backwards so we're
							// only converting one byte at a time.
							commandTemp[4 + i*2] = 0;
						}

						// Inform the user what we've done.
						sprintf(t, "c1SUBSTITUTED c9SENTc0:");
						for(i = 0; i < aLen; i++)
						{
							sprintf(t, "%s %.2x", t, aPacket[i]);
						}
						server->GamePrintInfo(t);

						returnLength = aLen;
					}
					else
					{
						// Execute the game command.
						server->GameCommandLine(commandTemp);
					}

					// Move on to the next command.
					strcpy(commandTemp, ptrSeparator);

					// Search for the next command separator.
					ptrSeparator = strchr(commandTemp, cCommandSeparator);
				}

				// Executed all of the commands except the last one ... so do that
				// one now.
				if( strcmp(commandTemp, "block") == 0 )
				{
					// This is the block command so do not send/receive the packet.
					sprintf(t, "c1BLOCKED c9SENTc0:");
					for(i = 0; i < aLen; i++)
					{
						sprintf(t, "%s %.2x", t, aPacket[i]);
					}
					server->GamePrintString(t);

					returnLength = 0;
				}
				// Is this the substitute command?
				else if( strnicmp(commandTemp, "sub", 3) == 0 )
				{

					// Sanity check.
					if( strlen(commandTemp) < 5)
					{
						sprintf(t,"c1Errorc0 You must specify a packet to substitute for the original.");
						server->GamePrintString(t);
						return aLen;
					}

					if( strlen(&commandTemp[4])/2 != aLen)
					{
						sprintf(t,"c1Errorc0 You must specify a packet of equal length to be substituted for the original.");
						server->GamePrintString(t);
						return aLen;
					}

					// This is the substitute command so 
					// use a different packet in place of the 
					// original one.

					// First, put any XX, YY, or ZZ values in there.
					substituteValues(&commandTemp[4]);

					// Now put the alternate packet in place of the real one.
					for( i=aLen-1 ; i >= 0 ; i-- )
					{
						//Convert the packet into binary format from back to front
						// and overwrite the original one.
						char *x="";
						aPacket[i] = (BYTE)strtoul(&commandTemp[4 + i*2], &x, 0x10);

						// Trim the packet from the end backwards so we're
						// only converting one byte at a time.
						commandTemp[4 + i*2] = 0;
					}

					// Inform the user what we've done.
					sprintf(t, "c1SUBSTITUTED c9SENTc0:");
					for(i = 0; i < aLen; i++)
					{
						sprintf(t, "%s %.2x", t, aPacket[i]);
					}
					server->GamePrintInfo(t);

					returnLength = aLen;
				}
				else
				{
					// Execute the game command.
					server->GameCommandLine(commandTemp);
				}
			}
		}
	}

	return returnLength;
}
